/*
    Copyright (C) 2007 by Michael Dale Long
  
    This file is part of DuSTED.
  
    DuSTED is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
  
    DuSTED is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License
    along with DuSTED.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef CANVAS_H
#define CANVAS_H

#include "widget.h"
#include "array.h"

#define CANVAS_WIDTH 8
#define CANVAS_HEIGHT 8

class Palette;

// Struct for holding points in Canvas::mPoints
struct Point {
	Point(void) { x = 0; y = 0; }
	Point(uint16 x, uint16 y) { this->x = x; this->y = y; }
	uint16 x;
	uint16 y;

	bool operator== (const Point &pt) const { return (pt.x == this->x && pt.y == this->y); }
};


class Canvas : public Widget {
	WIDGET_DECL(Canvas);

protected:
	// Reference rect for mapping canvas pixels to screen pixels
	Rect mTilePixelSize;
	// Reference rect similar to mTilePixelSize but used exclusively by Canvas::Render()
	Rect mRenderTile;
	// List of points that have changed and need to be rendered
	FixedArray<Point, 64> mPoints;
	// Current tile
	int mCurTile;

	void UpdatePixels(void);

public:
	// Holds the pixels for the canvas
	uint8 *mPixels;
	
	virtual void Init(void);
	virtual void HandleTouch(uint32 state, int16 x, int16 y);
	virtual void Render(void);

	void SetArea(int x, int y, int w, int h);

	void Refresh(void);

	void SetColor(uint16 x, uint16 y, uint16 color);

	inline int GetTile(void) const { return mCurTile; }
	void SetTile(int index);

	void PrevTile(void);
	void NextTile(void);
};

#endif // ! CANVAS_H

