/*
    Copyright (C) 2007 by Michael Dale Long
  
    This file is part of DuSTED.
  
    DuSTED is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
  
    DuSTED is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License
    along with DuSTED.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ENGINE_H
#define ENGINE_H

#include "array.h"

class Window;

// Size of persistent buffer (should be equal to size of SRAM, 64K)
#define PBUFFER_LEN 65536

// The main engine class
class Engine {
public:
	// Window focus stack
	DynArray<Window *> mFocusStack;
	// Buffer for data read/written to SRAM
	char *mPBuffer;
	// True if a homebrew cart is inserted
	bool mCartIsValid;

	// Initialize the engine
	void Init(void);
  
	// The main loop
	void Loop(void);

	// VBLANK Interrupt handler
	void VBlankHandler(void);


	// Read a section of the save buffer from SRAM.  By default, read the entire SRAM range.
	bool ReadPBuffer(int start = 0, int len = PBUFFER_LEN);

	// Write a section of the save buffer to SRAM.  By default, write the entire SRAM range.
	bool WritePBuffer(int start = 0, int len = PBUFFER_LEN);

	// Check to see if the SRAM is formatted properly (has the correct signature and version number)
	bool IsSRAMFormatted(void);

	// Format the SRAM
	bool FormatSRAM(void);

private:
	// 1-byte memcpy used for accessing GBA cart
	static void memcpy8(char *dest, const char *src, int len);
};

// Global engine instance
extern Engine *gEngine;

#endif // ENGINE_H
