/*
    Copyright (C) 2007 by Michael Dale Long
  
    This file is part of DuSTED.
  
    DuSTED is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
  
    DuSTED is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License
    along with DuSTED.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <nds.h>
#include "dusted.h"
#include "engine.h"
#include "palette.h"

WIDGET_IMPL(Palette, CID_PALETTE);

void Palette::Init(void) {
	// Set the palette to the appropriate space in the pbuffer
	mPalette = (uint16 *) (gEngine->mPBuffer + 64);
	// Initialize indices
	mPalIndex = 0;
	mPalPage = 0;

	if (!gEngine->mCartIsValid) {
		// Create hard-coded palette
		int rgb[3];
		int rgbindex = 0;
		mPalette[0] = RGB15(255, 0, 255); // Hot pink for invisible index
		mPalette[1] = RGB15(0, 0, 0); // Black
		mPalette[2] = RGB15(255, 255, 255); // White
		mPalette[3] = RGB15(255, 0, 0); // Red
		mPalette[4] = RGB15(0, 255, 0); // Green
		mPalette[5] = RGB15(0, 0, 255); // Blue
		mPalette[6] = RGB15(255, 255, 0); // Yellow
		mPalette[7] = RGB15(0, 255, 255); // Cyan
		for (int i = 8; i < PALETTE_SIZE; i++) {
			mPalette[i] = RGB15(rgb[0], rgb[1], rgb[2]);
			rgb[rgbindex] = (rgb[rgbindex] + 10) % 255;
			if (++rgbindex > 2) {
				rgbindex = 0;
			}
		}
	}

	// Let the palette render
	mVisible = true;
}


void Palette::HandleTouch(uint32 state, int16 x, int16 y) {
	if (state & PS_JUSTDOWN) {
		if (y < 160) { // If we're in the palette area...
			// Set the current palette index
			int16 px = (x - 192) / 16;
			int16 py = y / 16;
			mPalIndex = mPalPage * PALETTE_PAGE_SIZE + (px + py * 4);
		} else if (y > 180) {
			// If we're in the palette page area...
			if (x < 216) {
				// Page left
				mPalPage--;
				if (mPalPage < 0) {
					mPalPage = 5;
				}
			} else if (x > 232) {
				// Page right
				mPalPage++;
				if (mPalPage > 5) {
					mPalPage = 0;
				}
			}
		}
	}
}


void Palette::Render(void) {
	register uint16 color;
	register int i;
	Rect rect;
	// Render all the palette colors
	//TODO Change this so it only renders when palette colors change
	for (i = 0; i < PALETTE_PAGE_SIZE; i++) {
		// Draw the solid palette color
		rect.Set(mArea.x + ((i % 4) * 16) + 1, mArea.y + ((i / 4) * 16) + 1, 14, 14);
		rect.DrawFilled(mPalette[i + mPalPage * PALETTE_PAGE_SIZE], VRAM_A);

		// Select the outline color
		if (i + mPalPage * PALETTE_PAGE_SIZE == mPalIndex) {
			// White for the current color
			color = RGB15(255, 255, 255);
		} else {
			// Black for everything else
			color = 0;
		}

		// Draw the outline
		rect.x--;
		rect.y--;
		rect.w = 16;
		rect.h = 16;
		rect.DrawOutline(color, VRAM_A);
	}

	// Draw the page arrows.  For now we just use color blocks to represent them.
	//TODO write arrow drawing func
	rect.Set(mArea.x, mArea.y + 180, 24, 12);
	rect.DrawFilled(RGB15(127, 0, 0), VRAM_A);
	rect.x += 40;
	rect.DrawFilled(RGB15(0, 0, 127), VRAM_A);
}

