/*
    Copyright (C) 2007 by Michael Dale Long
  
    This file is part of DuSTED.
  
    DuSTED is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
  
    DuSTED is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License
    along with DuSTED.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <nds.h>
#include "dusted.h"
#include "canvas.h"
#include "undo.h"


void ActionHandler::Perform(const Action &action) {
	mRedoStack.RemoveAll();

	DoAction(action);

	if (mUndoStack.Size() == MAX_ACTIONS) {
		mUndoStack.Remove(MAX_ACTIONS - 1);
	}

	mUndoStack.Push(action);
}


bool ActionHandler::Undo(void) {
	if (mUndoStack.Size() == 0) {
		return false;
	}

	DoAction(mUndoStack[0], true);

	mRedoStack.Push(mUndoStack.Pop());

	return true;
}


bool ActionHandler::Redo(void) {
	if (mRedoStack.Size() == 0) {
		return false;
	}

	DoAction(mRedoStack[0], false);

	mUndoStack.Push(mRedoStack.Pop());

	return true;
}


void ActionHandler::DoAction(const Action &action, bool reverse) {
	switch (action.mCode) {
		case AC_NOOP: // No-op.  Doesn't do anything forward or reverse
			break;
		case AC_COLOR: // Set the color, to the old index if reverse, to the new index if not
			action.mColor.mCanvas->SetColor(action.mColor.mX, action.mColor.mY, (reverse? action.mColor.mOldIndex : action.mColor.mNewIndex));
			break;
		default: // Should never happen
			LogFatal("ActionHandler::DoAction():  Received unknown action code %d.\n", action.mCode);
	}
}

